package com.cweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.Base64Util;
import com.hai.common.security.VerifyCode;
import com.hai.common.security.VerifyCodeStorage;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.HttpsUtils;
import com.hai.common.utils.RedisUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.model.ResponseData;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Controller
@Api(value = "内容管理 内容发布")
@RequestMapping(value = "/sms")
public class SmsContentController {

    private static final String[] HEX_DIGITS = {"0" ,"1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"};

    private static Logger log = LoggerFactory.getLogger(CmsContentController.class);

    @Resource
    private RedisUtil redisUtil;

    @RequestMapping(value = "/sendSmsCode", method = RequestMethod.GET)
    @ApiOperation(value = "发送短信验证码")
    @ResponseBody
    public ResponseData sendSmsCode(@RequestParam(value = "phone", required = true) String phone) {
        try {
            VerifyCode verifyCode = VerifyCodeStorage.getDate(phone);
            String smsCode;
            if (verifyCode != null){
                smsCode = verifyCode.getObject();
            }else{
                // 生成随机6位验证码
                smsCode = String.valueOf(new Random().nextInt(899999) + 100000);
            }
            String param = "88200409698&to="+phone+"&templateId=f42261f78d54488391f1d1ac4bd5bd3f&templateParas=[\"" + smsCode +"\"]";
            Map<String,Object> body = new LinkedHashMap<>();
            body.put("from", param);
            String nonce = UUID.randomUUID().toString();
            String created = DateUtil.date2String(new Date(), "yyyy-MM-dd'T'HH:mm:ss'Z'");
            String passwordDigest = Base64Util.encode(encodeBySHA256(nonce + created + "9CWFv4NlaRX49n43VPA6eucZ8xeQ")) + "==";
            Map<String,Object> header = new LinkedHashMap<>();
            header.put("Authorization", "WSSE realm=\"SDP\",profile=\"UsernameToken\",type=\"Appkey\"");
            header.put("X-WSSE", "UsernameToken Username=\"QR5eMVrcW0Md6r634M6tf2i34cvb\",PasswordDigest=\""+passwordDigest+"\",Nonce=\""+nonce+"\",Created=\""+created+"\"");
            JSONObject object = HttpsUtils.doSmsPost("https://rtcsms.cn-north-1.myhuaweicloud.com:10743/sms/batchSendSms/v1", body,header);
            String code = object.getString("code");
            if (code.equals("000000")) {
                redisUtil.set(phone,smsCode, 60*10);
                return ResponseMsgUtil.success("发送成功");
            }
            return ResponseMsgUtil.success(object);
        } catch (Exception e) {
            log.error("CmsContentController --> sendSmsCode() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getSmsCode", method = RequestMethod.GET)
    @ApiOperation(value = "发送短信验证码")
    @ResponseBody
    public ResponseData getSmsCode(@RequestParam(value = "phone", required = true) String phone) {
        try {
            return ResponseMsgUtil.success(redisUtil.get(phone));
        } catch (Exception e) {
            log.error("CmsContentController --> getSmsCode() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    public String encodeBySHA256(String str) {
        try{
            MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
            messageDigest.reset();
            messageDigest.update(str.getBytes("UTF-8"));
            return getFormattedText(messageDigest.digest());
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return "";
    }

    private String getFormattedText(byte[] bytes) {
        int len = bytes.length;
        StringBuilder buf = new StringBuilder(len * 2);
        // 把密文转换成十六进制的字符串形式
        for (int j=0;j<len;j++){
            buf.append(HEX_DIGITS[(bytes[j] >> 4) & 0x0f]);
            buf.append(HEX_DIGITS[bytes[j] & 0x0f]);
        }
        return buf.toString();
    }
}
