package com.hai.service.pay.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.pay.util.XmlUtil;
import com.hai.common.pay.util.sdk.WXPayConstants;
import com.hai.common.utils.WxUtils;
import com.hai.config.CommonSysConst;
import com.hai.config.HuiLianTongConfig;
import com.hai.config.TuanYouConfig;
import com.hai.dao.HighCouponCodeOtherMapper;
import com.hai.dao.HighGasOrderPushMapper;
import com.hai.dao.HighUserCouponMapper;
import com.hai.entity.*;
import com.hai.enum_type.GoodsType;
import com.hai.enum_type.PayType;
import com.hai.model.HighMerchantStoreModel;
import com.hai.model.ResultProfitSharing;
import com.hai.service.*;
import com.hai.service.pay.NotifyService;
import com.hai.service.pay.PayService;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.net.ssl.SSLContext;
import java.io.FileInputStream;
import java.math.BigDecimal;
import java.security.KeyStore;
import java.util.*;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/27 00:35
 */
@Service("goodsOrderService")
public class GoodsOrderServiceImpl implements PayService {

    private static Logger log = LoggerFactory.getLogger(GoodsOrderServiceImpl.class);

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private HighGasOrderPushMapper highGasOrderPushMapper;

    @Resource
    private HighCouponService highCouponService;

    @Resource
    private HighCouponCodeService highCouponCodeService;

    @Resource
    private HighUserService highUserService;

    @Resource
    private HighUserCouponMapper highUserCouponMapper;

    @Resource
    private HighMerchantStoreService highMerchantStoreService;

    @Resource
    private HighProfitSharingRecordService highProfitSharingRecordService;

    @Resource
    private HighCouponCodeOtherMapper highCouponCodeOtherMapper;

    @Resource
    private NotifyService notifyService;

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void paySuccess(Map<String, String> map, String payType) throws Exception {
        if (payType.equals("Alipay")) {
            // 支付宝支付 todo 暂未开发
            return;
        }
        if (payType.equals("WechatPay")) {
            // 查询订单信息
            HighOrder order = highOrderService.getOrderByOrderNo(map.get("out_trade_no"));
            if (order == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_FOUND_ORDER, "");
            }

            // 查询用户
            HighUser highUser = highUserService.findByUserId(order.getMemId());

            if (order.getOrderStatus() == 1) {
                order.setPaySerialNo(map.get("transaction_id")); // 支付流水号
                order.setPayRealPrice(new BigDecimal(map.get("total_fee")).divide(new BigDecimal("100"))); // 实付金额
                order.setPayTime(new Date()); // 支付时间
                order.setPayModel(2);         // 支付模式：1 金币，2 第三方平台，3 混合
                order.setPayType(2);          // 支付方式： 1：支付宝  2：微信  3：金币
                order.setOrderStatus(2);      // 订单状态：1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消

                for (HighChildOrder highChildOrder : order.getHighChildOrderList()) {
                    // 商品类型   商品类型   1:卡卷   2:金币充值
                    if (highChildOrder.getGoodsType() == 1) {
                        highChildOrder.setChildOrdeStatus(2);  // 子订单状态：1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消

                        HighCoupon coupon = highCouponService.getCouponById(highChildOrder.getGoodsId());
                        // 贵州中石化
                        if (coupon.getCouponSource() == 4) {
                            // 获取token
                            JSONObject tokenObject = HuiLianTongConfig.getToken();
                            if (!tokenObject.getString("result").equals("success")) {
                                log.error("HighCouponController -> insertCoupon() error!","获取token失败");
                                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "获取token失败");
                            }
                            Map<String,Object> push = new HashMap<>();
                            push.put("token", tokenObject.getString("data"));
                            push.put("couTypeCode", coupon.getCouponKey());
                            push.put("distCouCount", highChildOrder.getSaleCount());
                            push.put("userPhone", highUser.getPhone());
                            push.put("thirdUserId", highUser.getUnionId());

                            // 推送给高速
                            JSONObject returnParam = HuiLianTongConfig.couJointDist(tokenObject.getString("data"), order.getOrderNo(),coupon.getCouponKey(), highChildOrder.getSaleCount(), highUser.getPhone(), highUser.getUnionId());
                            if (returnParam != null && returnParam.getString("result").equals("success")) {
                                JSONArray dataArray = returnParam.getJSONArray("data");
                                for (Object data : dataArray) {
                                    JSONObject dataObject = (JSONObject) data;
                                    HighCouponCodeOther couponCodeOther = new HighCouponCodeOther();
                                    couponCodeOther.setType(1);
                                    couponCodeOther.setOrderId(order.getId());
                                    couponCodeOther.setChildOrderId(highChildOrder.getId());
                                    couponCodeOther.setCouTypeCode(dataObject.getString("couTypeCode"));
                                    couponCodeOther.setCouNo(dataObject.getString("couNo"));
                                    couponCodeOther.setStatus(20);
                                    couponCodeOther.setCreateTime(new Date());
                                    couponCodeOther.setActiveTime(dataObject.getDate("activeTime"));
                                    couponCodeOther.setValidStartDate(dataObject.getDate("validStartDate"));
                                    couponCodeOther.setValidEndDate(dataObject.getDate("validEndDate"));
                                    highCouponCodeOtherMapper.insert(couponCodeOther);

                                    // 卡卷关联用户
                                    HighUserCoupon highUserCoupon = new HighUserCoupon();
                                    highUserCoupon.setMerchantId(coupon.getMerchantId());
                                    highUserCoupon.setCouponId(coupon.getId());
                                    highUserCoupon.setUserId(order.getMemId());
                                    highUserCoupon.setCreateTime(new Date());
                                    highUserCoupon.setQrCodeImg(dataObject.getString("couNo"));
                                    highUserCoupon.setUseEndTime(dataObject.getDate("validEndDate"));
                                    highUserCoupon.setStatus(1);  // 状态  0:已过期  1：未使用  2：已使用
                                    highUserCouponMapper.insert(highUserCoupon);
                                }
                            }

                            // 推送记录
                            HighGasOrderPush highGasOrderPush = new HighGasOrderPush();
                            highGasOrderPush.setCreateTime(new Date());
                            highGasOrderPush.setCode(returnParam.getString("result"));
                            highGasOrderPush.setRequestContent(JSONObject.toJSONString(push));
                            highGasOrderPush.setReturnContent(returnParam.toJSONString());
                            highGasOrderPushMapper.insert(highGasOrderPush);

                        } else {
                            HighCouponCode code = highCouponCodeService.getCouponCodeByOrderId(highChildOrder.getId());
                            code.setStatus(2);  // 状态：1.待销售 2.未使用 3.已使用  99.预支付
                            code.setReceiveTime(new Date());
                            highCouponCodeService.updateCouponCode(code);
                            // 卡卷关联用户
                            HighUserCoupon highUserCoupon = new HighUserCoupon();
                            highUserCoupon.setMerchantId(code.getMerchantId());
                            highUserCoupon.setCouponId(code.getCouponId());
                            highUserCoupon.setUserId(order.getMemId());
                            highUserCoupon.setCouponCodeId(code.getId());
                            highUserCoupon.setCreateTime(new Date());
                            highUserCoupon.setQrCodeImg(code.getExt1());

                            // 计算使用有效期
                            Calendar userEndTime = Calendar.getInstance();
                            userEndTime.setTime(new Date());
                            userEndTime.set(Calendar.HOUR_OF_DAY, 23);
                            userEndTime.set(Calendar.MINUTE, 59);
                            userEndTime.set(Calendar.SECOND, 59);
                            userEndTime.add(Calendar.DATE, coupon.getRecycleDay());
                            if (userEndTime.getTime().compareTo(code.getUseEndTime()) == 1) {
                                highUserCoupon.setUseEndTime(code.getUseEndTime());
                            } else {
                                highUserCoupon.setUseEndTime(userEndTime.getTime());
                            }
                            highUserCoupon.setStatus(1);  // 状态  0:已过期  1：未使用  2：已使用
                            highUserCouponMapper.insert(highUserCoupon);
                        }
                    }

                    if (highChildOrder.getGoodsType() == 2) {
                        highChildOrder.setChildOrdeStatus(3);  // 子订单状态：1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消
                        // 金币 1：100
                        Integer goldNum = new BigDecimal(highChildOrder.getTotalPrice().toString()).multiply(new BigDecimal("100")).intValue();
                        highUserService.goldHandle(highChildOrder.getGoodsId(), goldNum, 1, 1, highChildOrder.getId());
                    }

                    if (highChildOrder.getGoodsType() == 3) {
                        highChildOrder.setChildOrdeStatus(3);
                        HighMerchantStoreModel store = highMerchantStoreService.getMerchantStoreById(highChildOrder.getGoodsId());
                        // 推送团油订单
                        Map<String,Object> paramMap = new HashMap<>();
                        paramMap.put("gasId", store.getStoreKey());
                        paramMap.put("oilNo", highChildOrder.getGasOilNo());
                        paramMap.put("gunNo", highChildOrder.getGasGunNo());
                        BigDecimal priceGun = highChildOrder.getGasPriceGun();
                        BigDecimal priceVip = highChildOrder.getGasPriceVip();
                        paramMap.put("priceGun", priceGun);  // 枪单价
                        paramMap.put("priceVip", priceVip);  // 优惠价
                        paramMap.put("driverPhone", order.getMemPhone());
                        paramMap.put("thirdSerialNo", order.getOrderNo());
                        paramMap.put("refuelingAmount", highChildOrder.getTotalPrice());

                        // 油品类型 1:汽油:2:柴油;3:天然气
                        if (highChildOrder.getGasOilType() == 1) {
                            paramMap.put("accountNo", CommonSysConst.getSysConfig().getTuanYouGasolineAccount());
                        } else if (highChildOrder.getGasOilType() == 2) {
                            paramMap.put("accountNo", CommonSysConst.getSysConfig().getTuanYouDieselAccount());
                        }
                        JSONObject orderPushObject = TuanYouConfig.refuelingOrderPush(paramMap);
                        // 推送团油订单记录
                        HighGasOrderPush highGasOrderPush = new HighGasOrderPush();
                        highGasOrderPush.setCreateTime(new Date());
                        highGasOrderPush.setCode(orderPushObject.getString("code"));
                        highGasOrderPush.setRequestContent(JSONObject.toJSONString(paramMap));
                        highGasOrderPush.setReturnContent(orderPushObject.toJSONString());
                        highGasOrderPushMapper.insert(highGasOrderPush);

                        if (orderPushObject != null && orderPushObject.getString("code").equals("200")) {
                            highChildOrder.setGasOrderNo(orderPushObject.getJSONObject("result").getString("orderNo"));
                        }
                    }
                }

                highOrderService.updateOrder(order);

                new Thread(() -> {
                    try {
                        Thread.sleep(60*1000);
                        BigDecimal rake = new BigDecimal("0.01");

                        // 计算微信收取的手续费     支付金额 * 0.002      注：如果与两个相邻数字的距离相等，则为上舍入的舍入模式。
                        BigDecimal wxHandlingFee = order.getPayRealPrice().multiply(new BigDecimal("0.002")).setScale(2,BigDecimal.ROUND_HALF_DOWN);
                        BigDecimal price = order.getPayRealPrice().subtract(wxHandlingFee);

                        // 计算分账金额    手续费后的价格 * 0.01  注：如果与两个相邻数字的距离相等，则为上舍入的舍入模式。
                        BigDecimal profitSharingAmount = price.multiply(rake).setScale(2,BigDecimal.ROUND_DOWN);
                        this.wxProfitsharing(order.getPaySerialNo(),order.getOrderNo(),profitSharingAmount);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }).start();
            }
        }
    }


    public void wxProfitsharing(String transaction_id,String out_order_no, BigDecimal amount) {
        try {
            Map<String,String> param = new LinkedHashMap<>();
            param.put("appid", "wx637bd6f7314daa46");
            param.put("mch_id", "1289663601");
            param.put("sub_mch_id" , "1609882817"); // 个体户黎杨珍
            param.put("transaction_id" , transaction_id);
            param.put("out_order_no" , out_order_no);
            param.put("nonce_str" , WxUtils.makeNonStr());

            // 分账金额
            BigDecimal porofitSharingAmount = amount;

            List<Map<String,Object>> receiversList = new ArrayList<>();
            Map<String,Object> receiversMap = new LinkedHashMap<>();
            receiversMap.put("type", "MERCHANT_ID");
            receiversMap.put("account", "1603942866");
            receiversMap.put("amount", porofitSharingAmount.multiply(new BigDecimal("100")).intValue());
            receiversMap.put("description", "分给商户【惠昕石化】");
            receiversList.add(receiversMap);
            param.put("receivers" , JSONObject.toJSONString(receiversList));
            String signStr = WxUtils.generateSignature(param, "Skufk5oi85wDFGl888i6wsRSTkdd5df5" , WXPayConstants.SignType.HMACSHA256);
            param.put("sign" , signStr);
            String resultXmL = this.doRefundRequest(param.get("mch_id"),null, WxUtils.mapToXml(param));

            // 请求分账返回的结果
            ResultProfitSharing resultProfitSharing = XmlUtil.getObjectFromXML(resultXmL, ResultProfitSharing.class);

            HighProfitSharingRecord sharingRecord = new HighProfitSharingRecord();
            sharingRecord.setOutOrderNo(resultProfitSharing.getOut_order_no());
            sharingRecord.setTransactionId(resultProfitSharing.getTransaction_id());
            sharingRecord.setOrderId(resultProfitSharing.getOrder_id());
            sharingRecord.setStatus(resultProfitSharing.getResult_code());
            sharingRecord.setPrice(amount);
            sharingRecord.setCreateTime(new Date());
            sharingRecord.setContent(resultXmL);
            highProfitSharingRecordService.insert(sharingRecord);

        } catch (Exception e) {
            log.error("CmsContentController --> getCorporateAdvertising() error!", e);
        }
    }


    public CloseableHttpClient readCertificate(String mchId) throws Exception{
        /**
         * 注意PKCS12证书 是从微信商户平台-》账户设置-》 API安全 中下载的
         */
        KeyStore keyStore = KeyStore.getInstance("PKCS12");
        //P12文件目录 证书路径，这里需要你自己修改，linux下还是windows下的根路径
        FileInputStream instream = new FileInputStream("/home/project/wx_cert/1289663601_apiclient_cert.p12");
        // FileInputStream instream = new FileInputStream("G:\\hurui-project/hai-parent/hai-service/src/main/java/privatekey/1289663601_apiclient_cert.p12");
        try {
            keyStore.load(instream, "1289663601".toCharArray());//这里写密码..默认是你的MCHID
        } finally {
            instream.close();
        }
        SSLContext sslcontext = SSLContexts.custom().loadKeyMaterial(keyStore, "1289663601".toCharArray()).build();//这里也是写密码的
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslcontext, SSLConnectionSocketFactory.getDefaultHostnameVerifier());
        // Allow TLSv1 protocol only
        return HttpClients.custom().setSSLSocketFactory(sslsf).build();
    }

    public String doRefundRequest(String mchId, String url, String data) throws Exception {

        //小程序退款需要调用双向证书的认证
        CloseableHttpClient httpClient = readCertificate(mchId);
        try {
            HttpPost httpost = new HttpPost("https://api.mch.weixin.qq.com/secapi/pay/profitsharing"); // 设置响应头信息
            httpost.addHeader("Connection", "keep-alive");
            httpost.addHeader("Accept", "*/*");
            httpost.addHeader("Content-Type", "text/xml");
            httpost.addHeader("Host", "api.mch.weixin.qq.com");
            httpost.addHeader("X-Requested-With", "XMLHttpRequest");
            httpost.addHeader("Cache-Control", "max-age=0");
            httpost.addHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0) ");
            httpost.setEntity(new StringEntity(data, "UTF-8"));
            CloseableHttpResponse response = httpClient.execute(httpost);
            try {
                HttpEntity entity = response.getEntity();
                String jsonStr = EntityUtils.toString(response.getEntity(), "UTF-8");
                EntityUtils.consume(entity);
                return jsonStr;
            } finally {
                response.close();
            }
        } catch (Exception e){
            throw new RuntimeException(e);
        } finally {
            httpClient.close();
        }
    }
}
